/*
   FNT - SQL-backend for PostgreSQL - function connect

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <libpq-fe.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../include/global.h"

#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/sql.h"

#include "main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

DLLEXPORT fnt_sql_connect_t *
fntsql_connect (void)
{
    fnt_sql_connect_t *sql_connect;
    PGconn *conn;
    char *pghost;
    char *pgport;
    int pgport_len;

    if (connect_info.is_local_path)
    {
	pghost = connect_info.local_db_path;
	pgport = NULL;
    }
    else
    {
	pghost = connect_info.host_name;

	if (connect_info.port_number != 0)
	{
	    pgport = g_try_malloc (16);
	    if (pgport == NULL)
		return NULL;
	    sprintf (pgport, "%d", (void *) connect_info.port_number);

	}
	else
	{
	    pgport = NULL;
	}
    }

    conn = PQsetdbLogin (pghost,
			 pgport,
			 NULL,
			 NULL,
			 connect_info.db_name,
			 connect_info.user_name, connect_info.password);
    if (pgport != NULL)
	g_free (pgport);

    if (conn == NULL)
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Pgsql - fail to allocate memory"));
	return NULL;
    }

    if (PQstatus (conn) != CONNECTION_OK)
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Pgsql - fail to connect: '%s'"),
			PQerrorMessage (conn));
	PQfinish (conn);
	return NULL;
    }

    sql_connect = g_try_malloc (sizeof (fnt_sql_connect_t));
    if (sql_connect == NULL)
    {
	PQfinish (conn);
	return NULL;
    }
    sql_connect->db_handle = (void *) conn;

    sql_connect->info = g_try_malloc (sizeof (fnt_sql_connect_info_t));
    if (sql_connect->info == NULL)
    {
	PQfinish (conn);
	g_free (sql_connect);
	return NULL;
    }

    memcpy ((void *) sql_connect->info, (void *) &connect_info,
	    sizeof (fnt_sql_connect_info_t));


    if (connect_info.is_local_path)
    {
	sql_connect->info->local_db_path =
	    g_strdup (connect_info.local_db_path);
    }
    else
    {
	sql_connect->info->local_db_path = g_strdup ("");
    }

    sql_connect->info->db_name =
	g_strdup ((connect_info.db_name) ? connect_info.db_name : "");

    sql_connect->info->host_name =
	g_strdup ((connect_info.host_name) ? connect_info.host_name : "");

    sql_connect->info->user_name =
	g_strdup ((connect_info.user_name) ? connect_info.user_name : "");

    sql_connect->info->password =
	g_strdup ((connect_info.password) ? connect_info.password : "");

    return sql_connect;
}
